@@ -247,10 +247,11 @@ class GIRParser(object):
                             'set-value-func', 'get-value-func']:
                 func_name = node.attrib.get(_glibns(func_id))
                 obj.__dict__[func_id.replace('-', '_')] = func_name
-        self._namespace.append(obj)
 
         if self._types_only:
+            self._namespace.append(obj)
             return
+
         for iface in self._find_children(node, _corens('implements')):
             obj.interfaces.append(self._namespace.type_from_name(iface.attrib['name']))
         for iface in self._find_children(node, _corens('prerequisite')):
@@ -272,12 +273,14 @@ class GIRParser(object):
             func = self._parse_function_common(ctor, ast.Function, obj)
             func.is_constructor = True
             obj.constructors.append(func)
-        obj.fields.extend(self._parse_fields(node))
+        obj.fields.extend(self._parse_fields(node, obj))
         for prop in self._find_children(node, _corens('property')):
             obj.properties.append(self._parse_property(prop, obj))
         for signal in self._find_children(node, _glibns('signal')):
             obj.signals.append(self._parse_function_common(signal, ast.Signal, obj))
 
+        self._namespace.append(obj)
+
     def _parse_callback(self, node):
         callback = self._parse_function_common(node, ast.Callback)
         self._namespace.append(callback)
@@ -356,12 +359,12 @@ class GIRParser(object):
         self._namespace.track(func)
         return func
 
-    def _parse_fields(self, node):
+    def _parse_fields(self, node, obj):
         res = []
         names = (_corens('field'), _corens('record'), _corens('union'), _corens('callback'))
         for child in node.getchildren():
             if child.tag in names:
-                fieldobj = self._parse_field(child)
+                fieldobj = self._parse_field(child, obj)
                 res.append(fieldobj)
         return res
 
@@ -376,16 +379,18 @@ class GIRParser(object):
             compound.foreign = True
         self._parse_generic_attribs(node, compound)
         if not self._types_only:
-            compound.fields.extend(self._parse_fields(node))
+            compound.fields.extend(self._parse_fields(node, compound))
             for method in self._find_children(node, _corens('method')):
-                compound.methods.append(
-                    self._parse_function_common(method, ast.Function, compound))
+                func = self._parse_function_common(method, ast.Function, compound)
+                func.is_method = True
+                compound.methods.append(func)
             for func in self._find_children(node, _corens('function')):
                 compound.static_methods.append(
                     self._parse_function_common(func, ast.Function, compound))
             for ctor in self._find_children(node, _corens('constructor')):
-                compound.constructors.append(
-                    self._parse_function_common(ctor, ast.Function, compound))
+                func = self._parse_function_common(ctor, ast.Function, compound)
+                func.is_constructor = True
+                compound.constructors.append(func)
         return compound
 
     def _parse_record(self, node, anonymous=False):
@@ -480,9 +485,11 @@ class GIRParser(object):
                         get_type=node.attrib[_glibns('get-type')],
                         c_symbol_prefix=node.attrib.get(_cns('symbol-prefix')))
         self._parse_generic_attribs(node, obj)
-        self._namespace.append(obj)
+
         if self._types_only:
+            self._namespace.append(obj)
             return
+
         for method in self._find_children(node, _corens('method')):
             func = self._parse_function_common(method, ast.Function, obj)
             func.is_method = True
@@ -493,8 +500,9 @@ class GIRParser(object):
         for callback in self._find_children(node, _corens('callback')):
             obj.fields.append(
                 self._parse_function_common(callback, ast.Callback, obj))
+        self._namespace.append(obj)
 
-    def _parse_field(self, node):
+    def _parse_field(self, node, parent):
         type_node = None
         anonymous_node = None
         if node.tag in map(_corens, ('record', 'union')):
@@ -520,6 +528,7 @@ class GIRParser(object):
                       node.attrib.get('bits'),
                       anonymous_node=anonymous_node)
         field.private = node.attrib.get('private') == '1'
+        field.parent = parent
         self._parse_generic_attribs(node, field)
         return field
 
@@ -570,12 +579,14 @@ class GIRParser(object):
         obj.error_domain = glib_error_domain
         obj.ctype = ctype
         self._parse_generic_attribs(node, obj)
-        self._namespace.append(obj)
 
         if self._types_only:
+            self._namespace.append(obj)
             return
+
         for member in self._find_children(node, _corens('member')):
             members.append(self._parse_member(member))
         for func_node in self._find_children(node, _corens('function')):
             func = self._parse_function_common(func_node, ast.Function)
             obj.static_methods.append(func)
+        self._namespace.append(obj)
