@@ -125,7 +125,7 @@ usage is void (*_gtk_reserved1)(void);"""
     def _get_validate_parameter_name(self, parent, param_name, origin):
         try:
             param = parent.get_parameter(param_name)
-        except ValueError, e:
+        except ValueError:
             param = None
         if param is None:
             if isinstance(origin, ast.Parameter):
@@ -225,7 +225,7 @@ usage is void (*_gtk_reserved1)(void);"""
             section_name = 'SECTION:' + name.lower()
             block = self._blocks.get(section_name)
             if block:
-                node.doc = block.comment
+                node.doc = block.comment if block.comment else ''
         if isinstance(node, (ast.Class, ast.Interface)):
             for prop in node.properties:
                 self._apply_annotations_property(node, prop)
@@ -600,7 +600,7 @@ usage is void (*_gtk_reserved1)(void);"""
         if block is None:
             return
 
-        node.doc = block.comment
+        node.doc = block.comment if block.comment else ''
 
         since_tag = block.get_tag(TAG_SINCE)
         if since_tag is not None:
@@ -631,7 +631,7 @@ usage is void (*_gtk_reserved1)(void);"""
 
         annos_tag = block.get_tag(TAG_ATTRIBUTES)
         if annos_tag is not None:
-            for key, value in annos_tag.options.iteritems():
+            for key, value in annos_tag.options.items():
                 if value:
                     node.attributes.append((key, value.one()))
 
@@ -791,7 +791,7 @@ usage is void (*_gtk_reserved1)(void);"""
         block = self._blocks.get('%s::%s' % (prefix, signal.name))
         self._apply_annotations_annotated(signal, block)
         # We're only attempting to name the signal parameters if
-        # the number of parameter tags (@foo) is the same or greater
+        # the number of parameters (@foo) is the same or greater
         # than the number of signal parameters
         if block and len(block.params) > len(signal.parameters):
             names = block.params.items()
@@ -888,11 +888,10 @@ the ones that failed to resolve removed."""
                 else:
                     self._transformer.resolve_type(field.type)
         if isinstance(node, (ast.Class, ast.Interface)):
-            resolved_parent = None
             for parent in node.parent_chain:
                 try:
                     self._transformer.resolve_type(parent)
-                except ValueError, e:
+                except ValueError:
                     continue
                 target = self._transformer.lookup_typenode(parent)
                 if target:
@@ -1293,7 +1292,7 @@ method or constructor of some type."""
         params = node.parameters
 
         # First, do defaults for well-known callback types
-        for i, param in enumerate(params):
+        for param in params:
             argnode = self._transformer.lookup_typenode(param.type)
             if isinstance(argnode, ast.Callback):
                 if param.type.target_giname in ('Gio.AsyncReadyCallback',
@@ -1302,7 +1301,7 @@ method or constructor of some type."""
                     param.transfer = ast.PARAM_TRANSFER_NONE
 
         callback_param = None
-        for i, param in enumerate(params):
+        for param in params:
             argnode = self._transformer.lookup_typenode(param.type)
             is_destroynotify = False
             if isinstance(argnode, ast.Callback):
