@@ -267,6 +267,10 @@ void gi_marshalling_tests_utf8_full_inout (gchar **utf8);
 
 GSList *gi_marshalling_tests_filename_list_return (void);
 
+void gi_marshalling_tests_utf8_as_uint8array_in (const guint8 *array,
+                                                 gsize         len);
+
+
 /* Enum */
 
 typedef enum
@@ -434,6 +438,7 @@ void gi_marshalling_tests_garray_utf8_none_in (GArray *array_);
 void gi_marshalling_tests_garray_utf8_none_out (GArray **array_);
 void gi_marshalling_tests_garray_utf8_container_out (GArray **array_);
 void gi_marshalling_tests_garray_utf8_full_out (GArray **array_);
+void gi_marshalling_tests_garray_utf8_full_out_caller_allocated (GArray *array_);
 
 void gi_marshalling_tests_garray_utf8_none_inout (GArray **array_);
 void gi_marshalling_tests_garray_utf8_container_inout (GArray **array_);
@@ -711,6 +716,26 @@ struct _GIMarshallingTestsObjectClass
      */
     void (* method_int8_out) (GIMarshallingTestsObject *self, gint8 *out);
 
+    /**
+     * GIMarshallingTestsObjectClass::method_int8_arg_and_out_caller:
+     * @out: (out caller-allocates):
+     */
+    void (* method_int8_arg_and_out_caller) (GIMarshallingTestsObject *self, gint8 arg, gint8 *out);
+
+    /**
+     * GIMarshallingTestsObjectClass::method_int8_arg_and_out_callee:
+     * @out: (out):
+     */
+    void (* method_int8_arg_and_out_callee) (GIMarshallingTestsObject *self, gint8 arg, gint8 **out);
+
+    /**
+     * GIMarshallingTestsObjectClass::method_str_arg_out_ret:
+     * @out: (out caller-allocates):
+     *
+     * Returns: (transfer none)
+     */
+    const gchar* (* method_str_arg_out_ret) (GIMarshallingTestsObject *self, const gchar* arg, guint *out);
+
     /**
      * GIMarshallingTestsObjectClass::method_with_default_implementation:
      * @in: (in):
@@ -792,6 +817,9 @@ const gint *gi_marshalling_tests_object_method_array_return (GIMarshallingTestsO
 
 void gi_marshalling_tests_object_method_int8_in (GIMarshallingTestsObject *object, gint8 in);
 void gi_marshalling_tests_object_method_int8_out (GIMarshallingTestsObject *object, gint8 *out);
+void gi_marshalling_tests_object_method_int8_arg_and_out_caller (GIMarshallingTestsObject *object, gint8 arg, gint8 *out);
+void gi_marshalling_tests_object_method_int8_arg_and_out_callee (GIMarshallingTestsObject *object, gint8 arg, gint8 **out);
+const gchar* gi_marshalling_tests_object_method_str_arg_out_ret (GIMarshallingTestsObject *object, const gchar* arg, guint *out);
 void gi_marshalling_tests_object_method_with_default_implementation (GIMarshallingTestsObject *object, gint8 in);
 void gi_marshalling_tests_object_method_variant_array_in (GIMarshallingTestsObject *object, GVariant **in, gsize n_in);
 
